﻿using System.Collections.Generic;
using System.Windows;
using System.Windows.Input;
using System.Windows.Controls;
using System.Linq;

namespace FSNCleaner.Silverlight
{
    /// <summary>
    /// ViewModel de la fenetre principale
    /// </summary>
    public class MainPageViewModel : DependencyObject
    {
        #region Init
        /// <summary>
        /// Fichier à afficher
        /// </summary>
        public Fichier CurrentFile
        {
            get { return (Fichier)GetValue(CurrentFileProperty); }
            set { SetValue(CurrentFileProperty, value); }
        }
        public static readonly DependencyProperty CurrentFileProperty =
            DependencyProperty.Register("CurrentFile", typeof(Fichier), typeof(MainPageViewModel), new PropertyMetadata(null));

        /// <summary>
        /// Progression de l'opération
        /// </summary>
        public int Progress
        {
            get { return (int)GetValue(ProgressProperty); }
            set { SetValue(ProgressProperty, value); }
        }
        public static readonly DependencyProperty ProgressProperty =
            DependencyProperty.Register("Progress", typeof(int), typeof(MainPageViewModel), new PropertyMetadata(0));

        //Les commandes bindable à l'UI
        public ICommand CleanFileAllCommand { get; set; }
        public ICommand CleanFileWrapCommand { get; set; }
        public ICommand CleanFileLRCommand { get; set; }
        public ICommand CleanFileVoicesCommand { get; set; }
        public ICommand CleanFileCommentCommand { get; set; }
        public ICommand AddWordWrapCommand { get; set; }
        public ICommand AddLRCommand { get; set; }
        public ICommand AddBlankSpaceCommand { get; set; }
        public ICommand ReplaceDoubleQuoteCommand { get; set; }
        public ICommand ScriptFileCommand { get; set; }

        public MainPageViewModel()
        {
            this.CleanFileAllCommand = new CleanFileAllCommand(this);
            this.CleanFileWrapCommand = new CleanFileWrapCommand(this);
            this.CleanFileLRCommand = new CleanFileLRCommand(this);
            this.CleanFileVoicesCommand = new CleanFileVoicesCommand(this);
            this.CleanFileCommentCommand = new CleanFileCommentCommand(this);
            this.AddWordWrapCommand = new AddWordWrapCommand(this);
            this.AddLRCommand = new AddLRCommand(this);
            this.AddBlankSpaceCommand = new AddBlankSpaceCommand(this);
            this.ReplaceDoubleQuoteCommand = new ReplaceDoubleQuoteCommand(this);
            this.ScriptFileCommand = new ScriptFileCommand(this);
        }
        #endregion

        #region Menu Edit
        /// <summary>
        /// Nettoie un fichier script de tous les types de scripts et balises
        /// </summary>
        public void CleanFileAll()
        {
                CurrentFile.Clean();
        }

        /// <summary>
        /// Nettoie un fichier script de toutes les balises "[wrap text="..."]"
        /// </summary>
        public void CleanFileWrap()
        {
                FctRegex.EraseWrap(CurrentFile);
        }

        /// <summary>
        /// Nettoie un fichier script de toutes les balises "[l][r]"
        /// </summary>
        public void CleanFileLR()
        {
                FctRegex.EraseLR(CurrentFile);
        }

        /// <summary>
        /// Nettoie un fichier script de tous les scripts "@say"
        /// </summary>
        public void CleanFileVoices()
        {
                FctRegex.EraseSay(CurrentFile);
        }

        /// <summary>
        /// Nettoie un fichier script de tous les commentaires ";..."
        /// </summary>
        public void CleanFileComment()
        {
                FctRegex.EraseComment(CurrentFile);
        }

        /// <summary>
        /// Ajoute des balises "[wrap text="..."]" à un fichier
        /// </summary>
        public void AddWordWrap()
        {
                //On transforme le texte du Fichier en un tableau de lignes pour son traitement
                CurrentFile.MakeList();
                //On ajoute les balises [wrap text="..."] à ces lignes
                Scriptage.DoWordWrap(CurrentFile.Lines);
                //On retransforme le tableau de lignes en un texte sur une seule chaine de caractère
                CurrentFile.MakeString();
        }

        /// <summary>
        /// Ajoute des balises "[l][r]" à un fichier
        /// </summary>
        public void AddLR()
        {
                //On transforme le texte du Fichier en un tableau de lignes pour son traitement
                CurrentFile.MakeList();
                //On ajoute les balises "[l][r]" à ces lignes
                Scriptage.DoLR(CurrentFile.Lines);
                //On retransforme le tableau de lignes en un texte sur une seule chaine de caractère
                CurrentFile.MakeString();
        }

        /// <summary>
        /// Ajoute des espaces blancs là où manifestement il en manque
        /// </summary>
        public void AddBlankSpace()
        {
                //On transforme le texte du Fichier en un tableau de lignes pour son traitement
                CurrentFile.MakeList();
                //On ajoute deux espace aux lignes de text qui n'en ont pas
                List<int> linesModified = Scriptage.DoBlankSpace(CurrentFile.Lines);
                //On retransforme le tableau de lignes en un texte sur une seule chaine de caractère
                CurrentFile.MakeString();

                string message;
                if (linesModified.Count == 0)
                {
                    message = "No missing blank space found.";
                }
                else
                {
                    message = string.Format("{0} missing blanks added at lines: ", linesModified.Count);
                    foreach (int line in linesModified)
                    {
                        message += string.Format("\n- {0}: {1}", line, CurrentFile.Lines[line]);
                    }
                }
                MessageBox.Show(message);
        }

        /// <summary>
        /// Remplace d'un fichier de script de tous les doubles quotes par des guillemets
        /// </summary>
        public void ReplaceDoubleQuote()
        {
                FctRegex.ReplaceDoubleQuote(CurrentFile);
        }

        /// <summary>
        /// Scripte un fichier texte traduit à partir du fichier original correspondant
        /// </summary>
        public void ScriptFile()
        {
            bool filesIsCorrect = false;

            //On déclare les Fichiers dont nous aurons besoin
            Fichier TranslatedFile;
            Fichier OriginalFile;

            //On considère que le fichier chargé dans l'appli est le fichier traduit
            TranslatedFile = CurrentFile;

            //Il nous reste donc le fichier original à charger
            OpenFileDialog ofd = new OpenFileDialog();

            ofd.Filter = "Kirikiri Script|*.ks";
            ofd.Multiselect = false;

            if ((bool)ofd.ShowDialog())
            {
                try
                {
                    OriginalFile = new Fichier(ofd.File.OpenText());
                }
                catch
                {
                    //Si ça foire, on le dit et on jete le fichier inutilement ouvert
                    MessageBox.Show("Cannot read the Original file");
                    TranslatedFile = null;
                    return;
                }
            }
            else
            {
                TranslatedFile = null;
                return;
            }

            //Si les deux fichiers sont bien lu
            //On transforme le texte des Fichiers en un tableau de lignes pour leur traitement
            TranslatedFile.MakeList();
            OriginalFile.MakeList();

            //On vérifie que le nombre de ligne de texte traduit/à traduire correspondent,
            //cela permet de prévenir la plus part des mauvaises associations de fichiers,
            //car chaque fichier a un nombre variable de lignes à traduire.
            if ((TranslatedFile.Lines.Count(line => line.StartsWith(" "))) !=
                (OriginalFile.Lines.Count(line => line.StartsWith("["))))
            {
                if (MessageBox.Show("Translated file have " + (TranslatedFile.Lines.Count(line => line.StartsWith(" "))).ToString() +
                    " lines of text whereas Original file have " +
                    (OriginalFile.Lines.Count(line => line.StartsWith("["))).ToString() + " lines of text." +
                    "\nDo you want to continue?", "Warning", MessageBoxButton.OKCancel) == MessageBoxResult.OK)
                {
                    filesIsCorrect = true;
                }
            }
            else
            {
                filesIsCorrect = true;
            }

            //Si les fichiers sont chargés et de la bonne taille (ou qu'on le force)
            if (filesIsCorrect == true)
            {
                //On rajoute les balises neccessaire au bonne affichage du texte dans le jeu
                Scriptage.DoWordWrap(TranslatedFile.Lines);
                Scriptage.DoLR(TranslatedFile.Lines);

                //On remplace les lignes originales par celles traduites et balisées
                Scriptage.ReplaceText(OriginalFile, TranslatedFile);
                //On retransforme le tableau de lignes en un texte sur une seule chaine de caractère
                OriginalFile.MakeString();

                //On affiche le tout
                CurrentFile = OriginalFile;
            }
        }
        #endregion
    }
}
