﻿using System;
using System.Collections.Generic;
using System.Text.RegularExpressions;
using System.Linq;

namespace FSNCleaner
{
    /// <summary>
    /// Regroupement des fonctions d'expression régulière.
    /// </summary>
    public static class FctRegex
    {
        /// <summary>
        /// Efface toutes les lignes de script.
        /// </summary>
        /// <param name="MyFile">Fichier à nettoyer.</param>
        public static void EraseScript(this Fichier MyFile)
        {
            MyFile.TextString = Regex.Replace(MyFile.TextString, @"@.*\n", string.Empty);
        }

        /// <summary>
        /// Efface toutes les lignes de doublage.
        /// </summary>
        /// <param name="MyFile">Fichier à nettoyer.</param>
        public static void EraseSay(this Fichier MyFile)
        {
            MyFile.TextString = Regex.Replace(MyFile.TextString, @"@say.*\n", string.Empty);
        }

        /// <summary>
        /// Efface toutes les lignes de commentaire.
        /// </summary>
        /// <param name="MyFile">Fichier à nettoyer.</param>
        public static void EraseComment(this Fichier MyFile)
        {
            MyFile.TextString = Regex.Replace(MyFile.TextString, "^;.*\n", string.Empty, RegexOptions.Multiline);
        }

        /// <summary>
        /// Efface toutes les balises '[l][r]'.
        /// </summary>
        /// <param name="MyFile">Fichier à nettoyer.</param>
        public static void EraseLR(this Fichier MyFile)
        {
            MyFile.TextString = Regex.Replace(MyFile.TextString, "[[]l[]][[]r[]]", string.Empty);
        }

        /// <summary>
        /// Efface une couche de balises de Wrap.
        /// </summary>
        /// <param name="MyFile">Fichier à nettoyer.</param>
        public static void EraseWrap(this Fichier MyFile)
        {
            MyFile.TextString = Regex.Replace(MyFile.TextString, "[[]wrap text=\".*?\"[]]", string.Empty);
        }

        /// <summary>
        /// Efface toutes les balises '[r]'.
        /// </summary>
        /// <param name="MyFile">Fichier à nettoyer.</param>
        public static void EraseSlashR(this Fichier MyFile)
        {
            MyFile.TextString = Regex.Replace(MyFile.TextString, "[\r]", string.Empty);
        }

        /// <summary>
        /// Efface toutes les lignes 'pageX'.
        /// </summary>
        /// <param name="MyFile">Fichier à nettoyer.</param>
        public static void ErasePage(this Fichier MyFile)
        {
            MyFile.TextString = Regex.Replace(MyFile.TextString, @"[*].*\n", string.Empty);
        }

        /// <summary>
        /// Efface les balises '[lineX]'.
        /// </summary>
        /// <param name="MyFile">Fichier à nettoyer.</param>
        public static void EraseLineX(this Fichier MyFile)
        {
            MyFile.TextString = Regex.Replace(MyFile.TextString, @"[[]line.*[]]", string.Empty);
        }

        /// <summary>
        /// Compte le nombre de page d'un Fichier.
        /// </summary>
        /// <param name="MyFile">Fichier à traiter.</param>
        /// <returns>Nombre de pages du Fichier.</returns>
        public static Int32 CountPage(this Fichier MyFile)
        {
            MatchCollection MyMatchCollection = Regex.Matches(MyFile.TextString, "[*]page");

            return MyMatchCollection.Count;
        }

        /// <summary>
        /// Compte le nombre de mot de la liste de lignes d'un Fichier.
        /// </summary>
        /// <param name="MyFile">Fichier à traiter.</param>
        /// <returns>Nombre de pages du Fichiers.</returns>
        public static Int32 CountWord(this Fichier MyFile)
        {
            Regex myRegex = new Regex(@"\w");

            int nbWords = (from line in MyFile.Lines
                           from word in line.Split(null)
                           where myRegex.IsMatch(word)
                           select word).Count();

            return nbWords;
        }

        /// <summary>
        /// Remplace les apostrophes par des simples quotes.
        /// </summary>
        /// <param name="MyFile">Fichier à nettoyer.</param>
        public static void ReplaceApostrophe(this Fichier MyFile)
        {
            MyFile.TextString = Regex.Replace(MyFile.TextString, "‘", "'");
        }

        /// <summary>
        /// Remplace les doubles quotes par des guillemets.
        /// </summary>
        /// <param name="MyFile">Fichier à nettoyer.</param>
        public static void ReplaceDoubleQuote(this Fichier MyFile)
        {
            MyFile.TextString = Regex.Replace(MyFile.TextString, " \"", " « ");
            MyFile.TextString = Regex.Replace(MyFile.TextString, "\"", " »");
        }

        /// <summary>
        /// Remplace une balise type LineX par des tirets.
        /// </summary>
        /// <param name="line">Ligne à traiter.</param>
        /// <param name="regex">Expression régulière définissant la balise.</param>
        /// <param name="dash">Chaine de caractères contenant les tirets.</param>
        /// <returns>Ligne avec la balise word-wrapper.</returns>
        public static string ReplaceLineX(string line, string regex, string dash)
        {
            //On cherche toute les balises du type donnée
            MatchCollection ReplaceMatches = Regex.Matches(line, regex);

            //On efface la dernière balise trouvé
            line = line.Remove(ReplaceMatches[ReplaceMatches.Count - 1].Index,
                ReplaceMatches[ReplaceMatches.Count - 1].Length);
            //On insert les X tirets crée auparavant à la place
            line = line.Insert(ReplaceMatches[ReplaceMatches.Count - 1].Index, dash);

            return line;
        }

        /// <summary>
        /// Découpe les pages d'un texte en supprimant les balises *page.
        /// </summary>
        /// <param name="text">Le texte à découper.</param>
        /// <returns>La liste des pages.</returns>
        public static IEnumerable<string> SplitPage(string text)
        {
            return Regex.Split(text, @"[*].*\n");
        }

        /// <summary>
        /// Nettoye une liste de pages de tout caractère blanc ou de retour à la ligne.
        /// </summary>
        /// <param name="pages">Les pages à nettoyer.</param>
        /// <returns></returns>
        public static IList<string> CleanPages(IEnumerable<string> pages)
        {
            return pages.Select(page => Regex.Replace(page, @"[[][^]]*[]]", String.Empty)).Select(page => Regex.Replace(page, @"\W", String.Empty)).ToList();
        }
    }
}